@@ -175,6 +175,12 @@ public:
      */
     void addXMLGUIClient( KXMLGUIClient *client );
 
+    /**
+     * Removes an XML gui client that uses this toolbar
+     * @since 4.9.0
+     */
+    void removeXMLGUIClient( KXMLGUIClient *client );
+
     /**
      * Load state from an XML @param element, called by KXMLGUIBuilder.
      */
