@@ -105,7 +105,7 @@ static void appendRun( BidiState &bidi );
 
 static int getBPMWidth(int childValue, Length cssUnit)
 {
-    if (!cssUnit.isVariable())
+    if (!cssUnit.isAuto())
         return (cssUnit.isFixed() ? cssUnit.value() : childValue);
     return 0;
 }
@@ -1215,7 +1215,7 @@ void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator
             break;
             // neutrals
         case QChar::DirB:
-            // ### what do we do with newline and paragraph seperators that come to here?
+            // ### what do we do with newline and paragraph separators that come to here?
             break;
         case QChar::DirS:
             // ### implement rule L1
@@ -1367,7 +1367,7 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren, int breakBeforeLin
 
     m_height = borderTop() + paddingTop();
     int toAdd = borderBottom() + paddingBottom();
-    if (m_layer && scrollsOverflowX() && style()->height().isVariable())
+    if (m_layer && scrollsOverflowX() && style()->height().isAuto())
         toAdd += m_layer->horizontalScrollbarHeight();
 
     // Figure out if we should clear our line boxes.
@@ -1389,10 +1389,10 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren, int breakBeforeLin
         RenderObject *o = first( this, 0, false );
         while ( o ) {
             invalidateVerticalPosition();
-            if (o->markedForRepaint()) {
+            if (!fullLayout && o->markedForRepaint()) {
                 o->repaintDuringLayout();
                 o->setMarkedForRepaint(false);
-             }
+            }
             if (o->isReplaced() || o->isFloating() || o->isPositioned()) {
 
                 if ((!o->isPositioned() || o->isPosWithStaticDim()) && 
@@ -1411,8 +1411,10 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren, int breakBeforeLin
                 }
             }
             else {
-                if (fullLayout || o->selfNeedsLayout())
+                if (fullLayout || o->selfNeedsLayout()) {
                     o->dirtyInlineBoxes(fullLayout);
+                    o->setMarkedForRepaint(false);
+                }
                 o->setNeedsLayout(false);
             }
             o = Bidinext( this, o, 0, false );
@@ -1697,7 +1699,7 @@ RootInlineBox* RenderBlock::determineStartPosition(bool fullLayout, BidiIterator
             // We have a dirty line.
             if (RootInlineBox* prevRootBox = curr->prevRootBox()) {
                 // We have a previous line.
-                if (!prevRootBox->endsWithBreak() || prevRootBox->lineBreakObj()->isText() && prevRootBox->lineBreakPos() >= static_cast<RenderText*>(prevRootBox->lineBreakObj())->stringLength())
+                if (!prevRootBox->endsWithBreak() || (prevRootBox->lineBreakObj()->isText() && prevRootBox->lineBreakPos() >= static_cast<RenderText*>(prevRootBox->lineBreakObj())->stringLength()))
                     // The previous line didn't break cleanly or broke at a newline
                     // that has been deleted, so treat it as dirty too.
                     curr = prevRootBox;
@@ -1826,6 +1828,8 @@ static inline bool requiresLineBox(BidiIterator& it)
         return false;
     if (it.obj->isInlineFlow())
         return (getBorderPaddingMargin(it.obj, it.endOfInline) != 0);
+    if (it.obj->isText() && !static_cast<RenderText*>(it.obj)->length())
+        return false;
     if (it.obj->style()->preserveWS() || it.obj->isBR())
         return true;
 
@@ -1850,12 +1854,38 @@ bool RenderBlock::inlineChildNeedsLineBox(RenderObject* inlineObj) // WC: genera
 
     return !it.atEnd();
 }
+ 
+void RenderBlock::fitBelowFloats(int widthToFit, int& availableWidth)
+{
+    assert(widthToFit > availableWidth);
+
+    int floatBottom;
+    int lastFloatBottom = m_height;
+    int newLineWidth = availableWidth;
+    while (true) {
+        floatBottom = nearestFloatBottom(lastFloatBottom);
+        if (!floatBottom)
+            break;
+
+        newLineWidth = lineWidth(floatBottom);
+        lastFloatBottom = floatBottom;
+        if (newLineWidth >= widthToFit)
+            break;
+    }
+    if (newLineWidth > availableWidth) {
+        m_height = lastFloatBottom;
+        availableWidth = newLineWidth;
+#ifdef DEBUG_LINEBREAKS
+    kDebug(6041) << " new position at " << m_height << " newWidth " << availableWidth;
+#endif
+    }
+}
 
 BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi)
 {
     int width = lineWidth(m_height);
-    int w = 0;
-    int tmpW = 0;
+    int w = 0; // the width from the start of the line up to the currently chosen breaking opportunity
+    int tmpW = 0; // the accumulated width since the last chosen breaking opportunity
 #ifdef DEBUG_LINEBREAKS
     kDebug(6041) << "findNextLineBreak: line at " << m_height << " line width " << width;
     kDebug(6041) << "sol: " << start.obj << " " << start.pos;
@@ -1905,7 +1935,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         return start;
     }
 
-    // This variable says we have encountered an object after which initial whitespace should be ignored (e.g. InlineFlows at the begining of a line).
+    // This variable says we have encountered an object after which initial whitespace should be ignored (e.g. InlineFlows at the beginning of a line).
     // Either we have nothing to do, if there is no whitespace after the object... or we have to enter the ignoringSpaces state.
     // This dilemma will be resolved when we have a peek at the next object.
     bool checkShouldIgnoreInitialWhitespace = false;
@@ -1920,6 +1950,10 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
     // a run.
     bool currentCharacterIsSpace = false;
 
+    // This variable tracks whether there is space still available on the line for floating objects.
+    // Once a floating object does not fit, we wait till next linebreak before positioning more floats.
+    bool floatsFitOnLine = true;
+
     RenderObject* trailingSpaceObject = 0;
 
     BidiIterator lBreak = start;
@@ -1967,10 +2001,11 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 // check if it fits in the current line.
                 // If it does, position it now, otherwise, position
                 // it after moving to next line (in newLine() func)
-                if (o->width()+o->marginLeft()+o->marginRight()+w+tmpW <= width) {
+                if (floatsFitOnLine && o->width()+o->marginLeft()+o->marginRight()+w+tmpW <= width) {
                     positionNewFloats();
                     width = lineWidth(m_height);
-                }
+                } else
+                    floatsFitOnLine = false;
             }
             else if (o->isPositioned() && o->isPosWithStaticDim()) {
                 bool needToSetStaticX;
@@ -2035,7 +2070,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             currentCharacterIsSpace = false;
             trailingSpaceObject = 0;
 
-            if (o->isListMarker() && o->style()->listStylePosition() == OUTSIDE) {
+            if (o->isListMarker()) {
                 checkShouldIgnoreInitialWhitespace = true;
             }
         } else if ( o->isText() ) {
@@ -2054,6 +2089,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             int wordSpacing = o->style()->wordSpacing();
 #endif
             bool nextIsSoftBreakable = false;
+            bool checkBreakWord = autoWrap && (o->style()->wordWrap() == WWBREAKWORD);
 
             while(len) {
                 bool previousCharacterIsSpace = currentCharacterIsSpace;
@@ -2061,6 +2097,8 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 nextIsSoftBreakable = false;
                 const QChar c = str[pos];
                 currentCharacterIsSpace = c.unicode() == ' ';
+                checkBreakWord &= !w; // only break words when no other breaking opportunity exists earlier
+                                      // on the line (even within the text object we are currently processing)
 
                 if (preserveWS || !currentCharacterIsSpace)
                     isLineEmpty = false;
@@ -2113,9 +2151,8 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                         continue;
                     }
                 }
-
-                if ( (preserveLF && c.unicode() == '\n') || (autoWrap && (isBreakable( str, pos, strlen ) || isSoftBreakable)) ) {
-
+                bool isbreakablePosition = (preserveLF && c.unicode() == '\n') || (autoWrap && (isBreakable( str, pos, strlen ) || isSoftBreakable));
+                if ( isbreakablePosition || checkBreakWord) {
                     tmpW += t->width(lastSpace, pos - lastSpace, f);
 #ifdef APPLE_CHANGES
                     applyWordSpacing = (wordSpacing && currentCharacterIsSpace && !previousCharacterIsSpace &&
@@ -2124,31 +2161,18 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 #ifdef DEBUG_LINEBREAKS
                     kDebug(6041) << "found space at " << pos << " in string '" << QString( str, strlen ).toLatin1().constData() << "' adding " << tmpW << " new width = " << w;
 #endif
-                    if ( autoWrap && w + tmpW > width && w == 0 ) {
-                        int fb = nearestFloatBottom(m_height);
-                        int newLineWidth = lineWidth(fb);
-                        // See if |tmpW| will fit on the new line.  As long as it does not,
-                        // keep adjusting our float bottom until we find some room.
-                        int lastFloatBottom = m_height;
-                        while (lastFloatBottom < fb && tmpW > newLineWidth) {
-                            lastFloatBottom = fb;
-                            fb = nearestFloatBottom(fb);
-                            newLineWidth = lineWidth(fb);
-                        }
-
-                        if(!w && m_height < fb && width < newLineWidth) {
-                            m_height = fb;
-                            width = newLineWidth;
-#ifdef DEBUG_LINEBREAKS
-                            kDebug() << "RenderBlock::findNextLineBreak new position at " << m_height << " newWidth " << width;
-#endif
-                        }
-                    }
+                    if (!w && autoWrap && tmpW > width)
+                        fitBelowFloats(tmpW, width);
 
                     if (autoWrap) {
-                        if (w+tmpW > width)
+                        if (w+tmpW > width) {
+                            if (checkBreakWord && pos) {
+                                lBreak.obj = o;
+                                lBreak.pos = pos-1;
+                                lBreak.endOfInline = false;
+                            }
                             goto end;
-                        else if ( (pos > 1 && str[pos-1].unicode() == SOFT_HYPHEN) )
+                        } else if ( (pos > 1 && str[pos-1].unicode() == SOFT_HYPHEN) )
                             // Subtract the width of the soft hyphen out since we fit on a line.
                             tmpW -= t->width(pos-1, 1, f);
                     }
@@ -2164,7 +2188,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                         return lBreak;
                     }
 
-                    if ( autoWrap ) {
+                    if ( autoWrap && isbreakablePosition ) {
                         w += tmpW;
                         tmpW = 0;
                         lBreak.obj = o;
@@ -2207,23 +2231,40 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 len--;
             }
 
-            // IMPORTANT: pos is > length here!
-            if (!ignoringSpaces)
+            if (!ignoringSpaces) {
+                // We didn't find any space that would be beyond the line |width|.
+                // Lets add to |tmpW| the remaining width since the last space we found.
+                // Before we test this new |tmpW| however, we will have to look ahead to check
+                // if the next object/position can serve as a line breaking opportunity.
                 tmpW += t->width(lastSpace, pos - lastSpace, f);
-        } else
+                if (checkBreakWord && !w && pos && tmpW > width) {
+                    // Avoid doing the costly lookahead for break-word,
+                    // since we know we are allowed to break.
+                    lBreak.obj = o;
+                    lBreak.pos = pos-1;
+                    lBreak.endOfInline = false;
+                    goto end;
+                }
+            }
+        }
+        else {
             KHTMLAssert( false );
+        }
 
         InlineMinMaxIterator savedIt = lastIt;
         lastIt = it;
         o = it.next();
 
-        // advance the iterator to the next non-inline-flow
+        // Advance the iterator to the next non-inline-flow
         while (o && o->isInlineFlow() && !o->isWordBreak()) {
             tmpW += getBorderPaddingMargin(o, it.endOfInline);
             if (isLineEmpty) isLineEmpty = !tmpW;
             o = it.next();
         }
 
+        // All code below, until the end of the loop, is looking ahead the |it| object we just
+        // advanced to, comparing it to the previous object |lastIt|.
+
         if (checkShouldIgnoreInitialWhitespace) {
             // Check if we should switch to ignoringSpaces state
             if (!style()->preserveWS() && it.current && it.current->isText()) {
@@ -2243,26 +2284,38 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 
         bool checkForBreak = autoWrap;
         if (canBreak) {
-            if (w && w + tmpW > width && lBreak.obj && !lastIt.current->style()->preserveLF() && !autoWrap)
+            if (!autoWrap && w && w + tmpW > width && lBreak.obj && !lastIt.current->style()->preserveLF())
+                // ### needs explanation
                 checkForBreak = true;
             else if (it.current && lastIt.current->isText() && it.current->isText() && !it.current->isBR()) {
+                // We are looking ahead the next text object to see if it continues a word started previously,
+                // or is a line-breaking opportunity.
                 if (autoWrap || it.current->style()->autoWrap()) {
                     if (currentCharacterIsSpace)
+                        // "<i>s </i>top"
+                        //      _    ^
                         checkForBreak = true;
                     else {
+                        // either "<i>c</i>ontinue" or "<i>s</i> top"
+                        //            _    ^               _    ^
                         checkForBreak = false;
                         RenderText* nextText = static_cast<RenderText*>(it.current);
                         if (nextText->stringLength() != 0) {
                             QChar c = nextText->text()[0];
+                            // If the next item is a space, then we may try to break.
+                            // Otherwise the next text run continues our word (and so it needs to
+                            // keep adding to |tmpW|).
                             if (c == ' ' || c == '\t' || (c == '\n' && !it.current->style()->preserveLF())) {
-                                // If the next item on the line is text, and if we did not end with
-                                // a space, then the next text run continues our word (and so it needs to
-                                // keep adding to |tmpW|.  Just update and continue.
                                 checkForBreak = true;
                             }
                         }
 
-                        bool canPlaceOnLine = (w + tmpW <= width+1) || !autoWrap;
+                        bool willFitOnLine = (w + tmpW <= width);
+                        if (!willFitOnLine && !w) {
+                            fitBelowFloats(tmpW, width);
+                            willFitOnLine = tmpW <= width;
+                        }        
+                        bool canPlaceOnLine = willFitOnLine || !autoWrap;
                         if (canPlaceOnLine && checkForBreak) {
                             w += tmpW;
                             tmpW = 0;
@@ -2279,23 +2332,9 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                 if (currentCharacterIsSpace && !ignoringSpaces && !lastIt.current->style()->preserveWS())
                     trailingSpaceObject = 0;
 
-                int fb = nearestFloatBottom(m_height);
-                int newLineWidth = lineWidth(fb);
-                // See if |tmpW| will fit on the new line.  As long as it does not,
-                // keep adjusting our float bottom until we find some room.
-                int lastFloatBottom = m_height;
-                while (lastFloatBottom < fb && tmpW > newLineWidth) {
-                    lastFloatBottom = fb;
-                    fb = nearestFloatBottom(fb);
-                    newLineWidth = lineWidth(fb);
-                }
-                if( !w && m_height < fb && width < newLineWidth ) {
-                    m_height = fb;
-                    width = newLineWidth;
-#ifdef DEBUG_LINEBREAKS
-                kDebug() << "RenderBlock::findNextLineBreak new position at " << m_height << " newWidth " << width;
-#endif
-                }
+                if (w)
+                    goto end;
+                fitBelowFloats(tmpW, width);
 
                 // |width| may have been adjusted because we got shoved down past a float (thus
                 // giving us more room), so we need to retest, and only jump to
