@@ -55,7 +55,7 @@ void UploadDialog::Private::init()
     q->connect(atticaHelper, SIGNAL(contentLoaded(Attica::Content)), q, SLOT(_k_updatedContentFetched(Attica::Content)));
     q->connect(atticaHelper, SIGNAL(detailsLinkLoaded(QUrl)), q, SLOT(_k_detailsLinkLoaded(QUrl)));
     q->connect(atticaHelper, SIGNAL(currencyLoaded(QString)), q, SLOT(_k_currencyLoaded(QString)));
-    q->connect(atticaHelper, SIGNAL(previewLoaded(int, QImage)), q, SLOT(_k_previewLoaded(int, QImage)));
+    q->connect(atticaHelper, SIGNAL(previewLoaded(int,QImage)), q, SLOT(_k_previewLoaded(int,QImage)));
     atticaHelper->init();
 
     q->connect(ui.changePreview1Button, SIGNAL(clicked()), q, SLOT(_k_changePreview1()));
@@ -488,6 +488,28 @@ void UploadDialog::setVersion(const QString& version)
     d->ui.mVersionEdit->setText(version);
 }
 
+void UploadDialog::setPreviewImageFile(uint number, const KUrl& file)
+{
+    QPixmap preview(file.toLocalFile());
+    switch(number) {
+    case 0 :
+        d->previewFile1 = file;
+        d->ui.previewImage1->setPixmap(preview.scaled(d->ui.previewImage1->size()));
+        break;
+    case 1 :
+        d->previewFile2 = file;
+        d->ui.previewImage2->setPixmap(preview.scaled(d->ui.previewImage2->size()));
+        break;
+    case 2 :
+        d->previewFile3 = file;
+        d->ui.previewImage3->setPixmap(preview.scaled(d->ui.previewImage3->size()));
+        break;
+    default :
+        kError() << "Wrong preview image file number";
+        break;
+    }
+}
+
 void UploadDialog::Private::_k_priceToggled(bool priceEnabled)
 {
     ui.priceGroupBox->setEnabled(priceEnabled);
@@ -682,7 +704,7 @@ void UploadDialog::Private::doUpload(const QString& index, const KUrl& path)
 
     QString fileName = QFileInfo(path.toLocalFile()).fileName();
 
-    Attica::PostJob* job;
+    Attica::PostJob* job = 0;
     if (index.isEmpty()) {
         job = currentProvider().setDownloadFile(contentId, fileName, fileContents);
         q->connect(job, SIGNAL(finished(Attica::BaseJob*)), q, SLOT(_k_fileUploadFinished(Attica::BaseJob*)));
@@ -696,8 +718,8 @@ void UploadDialog::Private::doUpload(const QString& index, const KUrl& path)
         job = currentProvider().setPreviewImage(contentId, index, fileName, fileContents);
         q->connect(job, SIGNAL(finished(Attica::BaseJob*)), q, SLOT(_k_preview3UploadFinished(Attica::BaseJob*)));
     }
-
-    job->start();
+    if( job ) 
+      job->start();
 }
 
 void UploadDialog::Private::_k_fileUploadFinished(Attica::BaseJob* )
